unit hiImg_Rotate; {       } 

interface

uses Windows, Kol,Share,Debug;

type   TLine = array[0..0]of byte;
       PLine = ^TLine;
         
type
  THIImg_Rotate = class(TDebug)
    private
      Bitmap:PBitmap;
      RotateBmp:PBitmap;
    public
      _prop_angle:real;
      _prop_BackgroundColor:integer;
      _data_Bitmap:THI_Event;
      _data_angle:THI_Event;
      _data_BackgroundColor:THI_Event;
      _event_onResult:THI_Event;
      destructor Destroy ; override;
      procedure _work_doLoad(var _Data:TData; Index:word);
      procedure _work_doRotate(var _Data:TData; Index:word);
      procedure _work_doFlipVertical(var _Data:TData; Index:word);
      procedure _work_doFlipHorizontal(var _Data:TData; Index:word);
      procedure _var_Width(var _Data:TData; Index:word);
      procedure _var_Height(var _Data:TData; Index:word);
      procedure _var_Result(var _Data:TData; Index:word);      
    end;
implementation

destructor THIImg_Rotate.Destroy;
begin
   if Assigned(RotateBmp) then RotateBmp.Free;
   inherited;
end;

procedure THIImg_Rotate._work_doLoad;
begin
   Bitmap := ReadBitmap(_Data,_data_Bitmap,nil);
   if (Bitmap = nil) or Bitmap.Empty then exit;   
   Bitmap.PixelFormat := pf24bit;
   if not Assigned(RotateBmp) then RotateBmp := NewDIBBitmap(1, 1, pf24bit);
end;

procedure THIImg_Rotate._work_doRotate;
var   color : integer;
      angle : real;
      
      cosA, sinA: real;
      highest, lowest, mostleft, mostright: TPoint; 
      topoverh, leftoverh: integer; 
      x, y, k, H, W, newx, newy: integer;
      Sor,Des: PLine;

      function Point(x:Integer; y:Integer): TPoint;
      begin 
         Result.x := x; 
         Result.y := y; 
      end;

begin
   if (Bitmap = nil) or Bitmap.Empty then exit;

   angle := ReadReal(_Data,_data_angle,_prop_angle);
   color := ReadInteger(_Data,_data_BackgroundColor,_prop_BackgroundColor);

   while angle >= 360 do angle := angle - 360;
   while angle < 0 do angle := angle + 360;

   case Round(angle) of
          0: RotateBmp.Assign(Bitmap);
         90: begin
             RotateBmp.Assign(Bitmap);
             RotateBmp.RotateRightTrueColor;
             end;
        270: begin
             RotateBmp.Assign(Bitmap);
             RotateBmp.RotateLeftTrueColor;
             end;
        180: begin
             RotateBmp.Assign(Bitmap);
             RotateBmp.RotateRightTrueColor;
             RotateBmp.RotateRightTrueColor;
      end else begin
         angle := angle*Pi/180;
         H := Bitmap.Height;
         W := Bitmap.Width; 
         if (angle <= (pi / 2)) then begin 
            highest := Point(0,0);
            lowest := Point(W,H);
            mostleft := Point(0,H);
            mostright := Point(W,0);
         end else if (angle <= pi) then begin 
            highest := Point(0,H); 
            lowest := Point(W,0); 
            mostleft := Point(W,H); 
            mostright := Point(0,0); 
         end else if (angle <= (pi * 3 / 2)) then begin 
            highest := Point(W,H); 
            lowest := Point(0,0); 
            mostleft := Point(W, 0); 
            mostright := Point(0,H); 
         end else begin 
            highest := Point(W, 0); 
            lowest := Point(0,H); 
            mostleft := Point(0,0); 
            mostright := Point(W,H); 
         end;
         cosA := cos(angle);
         sinA := sin(angle);
         topoverh  := Round(highest.x * sinA + highest.y * cosA); 
         leftoverh := Round(mostleft.x * cosA - mostleft.y * sinA); 

         y := Abs(Round(lowest.x * sinA + lowest.y * cosA)) + Abs(topoverh) + 2; 
         x := Abs(Round(mostright.x * cosA - mostright.y * sinA)) + Abs(leftoverh) + 2;

         RotateBmp.free; 
         RotateBmp := NewDIBBitmap(x, y, pf24bit);
         RotateBmp.Canvas.Brush.Color := Color;
         FillRect(RotateBmp.Canvas.handle, RotateBmp.BoundsRect, RotateBmp.Canvas.Brush.Handle);
         for y := 0 to H - 1 do begin
            Sor := Bitmap.ScanLine[y];
            for x := 0 to W - 1 do begin
               newX := Abs(Round(x * cosA - y * sinA) - leftoverh) * 3; 
               newY := Round(x * sinA + y * cosA) - topoverh;
               k := x * 3;
               Des := RotateBmp.ScanLine[newY];
               Des[newx]   := Sor[k];   //B               
               Des[newx+1] := Sor[k+1]; //G
               Des[newx+2] := Sor[k+2]; //R
               Des := RotateBmp.ScanLine[newY + 1];
               Des[newx]   := Sor[k];   //B
               Des[newx+1] := Sor[k+1]; //G
               Des[newx+2] := Sor[k+2]; //R
            end;
         end;
      end;
   end;
   _hi_OnEvent(_event_onResult);
end;

procedure THIImg_Rotate._work_doFlipVertical;
begin
   if (Bitmap = nil) or Bitmap.Empty then exit;
   RotateBmp.Assign(Bitmap);
   RotateBmp.FlipVertical;
   _hi_onEvent(_event_onResult);
end;

procedure THIImg_Rotate._work_doFlipHorizontal;
begin
   if (Bitmap = nil) or Bitmap.Empty then exit;
   RotateBmp.Assign(Bitmap);
   RotateBmp.FlipHorizontal;
   _hi_onEvent(_event_onResult);
end;

procedure THIImg_Rotate._var_Width;
begin
   if (Bitmap = nil) or Bitmap.Empty then exit;
   dtInteger(_Data, RotateBmp.Width);
end;

procedure THIImg_Rotate._var_Height;
begin
   if (Bitmap = nil) or Bitmap.Empty then exit;
   dtInteger(_Data, RotateBmp.Height);
end;

procedure THIImg_Rotate._var_Result;
begin
   if (Bitmap = nil) or Bitmap.Empty then exit;
   dtBitmap(_Data, RotateBmp);
end;

end.
